@@ -4,8 +4,6 @@
  * appropriate bit map is searched for a free entry.
  *
  * The entry points into this file are
- *   load_bit_maps:   get the bit maps for the root or a newly mounted device
- *   unload_bit_maps: write the bit maps back to disk after an UMOUNT
  *   alloc_bit:       somebody wants to allocate a zone or inode; find one
  *   free_bit:        indicate that a zone or inode is available for allocation
  *   get_super:       search the 'superblock' table for a device
@@ -20,144 +18,82 @@
 #include "inode.h"
 #include "super.h"
 
-#define BITCHUNK_BITS (usizeof(bitchunk_t) * CHAR_BIT)
-#define BIT_MAP_SHIFT     13	/* (log2 of BLOCK_SIZE) + 3; 13 for 1k blocks*/
-
-/*===========================================================================*
- *				load_bit_maps				     *
- *===========================================================================*/
-PUBLIC int load_bit_maps(dev)
-dev_t dev;			/* which device? */
-{
-/* Load the bit map for some device into the cache and set up superblock. */
-
-  register int i;
-  register struct super_block *sp;
-  block_t zbase;
-
-  sp = get_super(dev);		/* get the superblock pointer */
-  if (bufs_in_use + sp->s_imap_blocks + sp->s_zmap_blocks >= NR_BUFS - 3)
-	return(EGENERIC);	/* insufficient buffers left for bit maps */
-  if (sp->s_imap_blocks > I_MAP_SLOTS || sp->s_zmap_blocks > Z_MAP_SLOTS)
-	return(EGENERIC);
-
-  /* Load the inode map from the disk. */
-  for (i = 0; i < sp->s_imap_blocks; i++)
-	sp->s_imap[i] = get_block(dev, SUPER_BLOCK + 1 + i, NORMAL);
-
-  /* Load the zone map from the disk. */
-  zbase = SUPER_BLOCK + 1 + sp->s_imap_blocks;
-  for (i = 0; i < sp->s_zmap_blocks; i++)
-	sp->s_zmap[i] = get_block(dev, zbase + i, NORMAL);
-
-  /* Inodes 0 and 1, and zone 0 are never allocated.  Mark them as busy. */
-  sp->s_imap[0]->b_bitmap[0] |= conv2(sp->s_native, 3); /* inodes 0, 1 busy */
-  sp->s_zmap[0]->b_bitmap[0] |= conv2(sp->s_native, 1); /* zone 0 busy */
-  return(OK);
-}
-
-
-
-/*===========================================================================*
- *				unload_bit_maps				     *
- *===========================================================================*/
-PUBLIC int unload_bit_maps(dev)
-dev_t dev;			/* which device is being unmounted? */
-{
-/* Unload the bit maps so a device can be unmounted. */
-
-  register int i;
-  register struct super_block *sp;
-
-  sp = get_super(dev);		/* get the superblock pointer */
-
-  for (i = 0; i < sp->s_imap_blocks; i++)
-	put_block(sp->s_imap[i],I_MAP_BLOCK);
-
-  for (i = 0; i < sp->s_zmap_blocks; i++)
-	put_block(sp->s_zmap[i], ZMAP_BLOCK);
-
-  return(OK);
-}
-
+#define BITCHUNK_BITS	(usizeof(bitchunk_t) * CHAR_BIT)
+#define BITS_PER_BLOCK	(BITMAP_CHUNKS * BITCHUNK_BITS)
 
 /*===========================================================================*
  *				alloc_bit				     *
  *===========================================================================*/
-PUBLIC bit_t alloc_bit(map_ptr, map_bits, bit_blocks, origin)
-struct buf *map_ptr[];		/* pointer to array of bit block pointers */
-bit_t map_bits;			/* how many bits are there in the bit map? */
-int bit_blocks;			/* how many blocks are there in the bit map? */
+PUBLIC bit_t alloc_bit(sp, map, origin)
+struct super_block *sp;		/* the filesystem to allocate from */
+int map;			/* IMAP (inode map) or ZMAP (zone map) */
 bit_t origin;			/* number of bit to start searching at */
 {
 /* Allocate a bit from a bit map and return its bit number. */
 
-  int i, o, w, w_off, b, count;
-  bit_t a;
-  register bitchunk_t *wptr, *wlim;
-  bitchunk_t k;
-  short block_count;
+  block_t start_block;		/* first bit block */
+  bit_t map_bits;		/* how many bits are there in the bit map? */
+  unsigned bit_blocks;		/* how many blocks are there in the bit map? */
+  unsigned block, word, bcount, wcount;
   struct buf *bp;
-  struct super_block *sp;
+  bitchunk_t *wptr, *wlim, k;
+  bit_t i, b;
 
-  sp = get_super(map_ptr[0]->b_dev);	/* get the superblock pointer */
-  if (sp->s_rd_only) panic("can't allocate bit on read-only filesys.", NO_NUM);
-  
-  /* Figure out where to start the bit search (depends on 'origin'). */
-  if (origin < 0 || origin >= map_bits) origin = 0;	/* for robustness */
+  if (sp->s_rd_only)
+	panic("can't allocate bit on read-only filesys.", NO_NUM);
 
-  /* Truncation of the next expression from a bit_t to an int is safe because
-   * it it is inconceivable that the number of blocks in the bit map > 32K.
-   */  
-  b = (int) (origin >> BIT_MAP_SHIFT);	/* relevant bit map block. */
-
-  /* Truncation of the next expression from a bit_t to an int is safe because
-   * its value is smaller than BITS_PER_BLOCK and easily fits in an int.
-   * The expression is better written as (int) origin % BITS_PER_BLOCK or
-   * even (int) origin & ~(BITS_PER_BLOCK - 1).
-   */  
-  o = (int) (origin - ((bit_t) b << BIT_MAP_SHIFT) );
-  w = o/BITCHUNK_BITS;
-  block_count = (w == 0 ? bit_blocks : bit_blocks + 1);
-
-  /* The outer while loop iterates on the blocks of the map.  The inner
-   * while loop iterates on the words of a block.  The for loop iterates
-   * on the bits of a word.
-   */
-  while (block_count--) {
-	/* If need be, loop on all the blocks in the bit map. */
-	bp = map_ptr[b];
-	wptr = &bp->b_bitmap[w];
-	wlim = &bp->b_bitmap[BITMAP_CHUNKS];
-	count = 0;
-	while (count < BITMAP_CHUNKS) {
-		/* Loop on all the words of one of the bit map blocks. */
-		if (*wptr != 0xFFFF) {
-			/* This word contains a free bit.  Allocate it. */
-			k = conv2(sp->s_native, (int) *wptr);
-			for (i = 0; i < BITCHUNK_BITS; i++)
-				if (((k >> i) & 1) == 0) {
-					w_off = (int)(wptr - &bp->b_bitmap[0]);
-					w_off = w_off * BITCHUNK_BITS;
-					a = i + w_off
-					    + ((bit_t) b << BIT_MAP_SHIFT);
-					/* If 'a' beyond map check other blks*/
-					if (a >= map_bits) {
-						wptr = wlim - 1;
-						break;
-					}
-					k |= 1 << i;
-					*wptr = conv2(sp->s_native, (int) k);
-					bp->b_dirt = DIRTY;
-					return(a);
-				}
-		}
-		if (++wptr == wlim) wptr = &bp->b_bitmap[0];	/* wrap */
-		count++;
-	}
-	if (++b == bit_blocks) b = 0;	/* we have wrapped around */
-	w = 0;
+  if (map == IMAP) {
+	start_block = SUPER_BLOCK + 1;
+	map_bits = sp->s_ninodes + 1;
+	bit_blocks = sp->s_imap_blocks;
+  } else {
+	start_block = SUPER_BLOCK + 1 + sp->s_imap_blocks;
+	map_bits = sp->s_zones - (sp->s_firstdatazone - 1);
+	bit_blocks = sp->s_zmap_blocks;
   }
+
+  /* Figure out where to start the bit search (depends on 'origin'). */
+  if (origin >= map_bits) origin = 0;	/* for robustness */
+
+  /* Locate the starting place. */
+  block = origin / BITS_PER_BLOCK;
+  word = (origin % BITS_PER_BLOCK) / BITCHUNK_BITS;
+
+  /* Iterate over all blocks plus one, because we start in the middle. */
+  bcount = bit_blocks + 1;
+  do {
+	bp = get_block(sp->s_dev, start_block + block, NORMAL);
+	wlim = &bp->b_bitmap[BITMAP_CHUNKS];
+
+	/* Iterate over the words in block. */
+	for (wptr = &bp->b_bitmap[word]; wptr < wlim; wptr++) {
+
+		/* Does this word contain a free bit? */
+		if (*wptr == (bitchunk_t) ~0) continue;
+
+		/* Find and allocate the free bit. */
+		k = conv2(sp->s_native, (int) *wptr);
+		for (i = 0; (k & (1 << i)) != 0; ++i) {}
+
+		/* Bit number from the start of the bit map. */
+		b = ((bit_t) block * BITS_PER_BLOCK)
+		    + (wptr - &bp->b_bitmap[0]) * BITCHUNK_BITS
+		    + i;
+
+		/* Don't allocate bits beyond the end of the map. */
+		if (b >= map_bits) break;
+
+		/* Allocate and return bit number. */
+		k |= 1 << i;
+		*wptr = conv2(sp->s_native, (int) k);
+		bp->b_dirt = DIRTY;
+		put_block(bp, MAP_BLOCK);
+		return(b);
+	}
+	put_block(bp, MAP_BLOCK);
+	if (++block >= bit_blocks) block = 0;	/* last block, wrap around */
+	word = 0;
+  } while (--bcount > 0);
   return(NO_BIT);		/* no bit could be allocated */
 }
 
@@ -165,38 +101,44 @@ bit_t origin;			/* number of bit to start searching at */
 /*===========================================================================*
  *				free_bit				     *
  *===========================================================================*/
-PUBLIC void free_bit(map_ptr, bit_returned)
-struct buf *map_ptr[];		/* pointer to array of bit block pointers */
+PUBLIC void free_bit(sp, map, bit_returned)
+struct super_block *sp;		/* the filesystem to operate on */
+int map;			/* IMAP (inode map) or ZMAP (zone map) */
 bit_t bit_returned;		/* number of bit to insert into the map */
 {
 /* Return a zone or inode by turning off its bitmap bit. */
 
-  int b, r, w, bit;
+  unsigned block, word, bit;
   struct buf *bp;
-  bitchunk_t k;
-  struct super_block *sp;
+  bitchunk_t k, mask;
+  block_t start_block;
 
-  sp = get_super(map_ptr[0]->b_dev);	/* get the superblock pointer */
-  if (sp->s_rd_only) panic("can't free bit on read-only file system", NO_NUM);
+  if (sp->s_rd_only)
+	panic("can't free bit on read-only filesys.", NO_NUM);
 
-  /* The truncations in the next two assignments are valid by the same
-   * reasoning as in alloc_bit.
-   */
-  b = (int) (bit_returned >> BIT_MAP_SHIFT);	/* which block it is in */
-  r = (int) (bit_returned - ((bit_t) b << BIT_MAP_SHIFT) );
-  w = r/BITCHUNK_BITS;		/* 'w' tells which word it is in */
-  bit = r % BITCHUNK_BITS;
-  bp = map_ptr[b];
-  if (bp == NIL_BUF) return;
-
-  k = conv2(sp->s_native, (int) bp->b_bitmap[w]);
-  if (((k >> bit) & 1) == 0) {
-	printf("Cannot free bit %ld\n", bit_returned);
-	panic("freeing unused block or inode--check file sys", NO_NUM);
+  if (map == IMAP) {
+	start_block = SUPER_BLOCK + 1;
+  } else {
+	start_block = SUPER_BLOCK + 1 + sp->s_imap_blocks;
   }
-  k &= ~(1 << bit);	/* turn the bit off */
-  bp->b_bitmap[w] = conv2(sp->s_native, (int) k);
+  block = bit_returned / BITS_PER_BLOCK;
+  word = (bit_returned % BITS_PER_BLOCK) / BITCHUNK_BITS;
+  bit = bit_returned % BITCHUNK_BITS;
+  mask = 1 << bit;
+
+  bp = get_block(sp->s_dev, start_block + block, NORMAL);
+
+  k = conv2(sp->s_native, (int) bp->b_bitmap[word]);
+  if (!(k & mask)) {
+	panic(map == IMAP ? "tried to free unused inode" :
+	      "tried to free unused block", NO_NUM);
+  }
+
+  k &= ~mask;
+  bp->b_bitmap[word] = conv2(sp->s_native, (int) k);
   bp->b_dirt = DIRTY;
+
+  put_block(bp, MAP_BLOCK);
 }
 
 
