@@ -9,7 +9,7 @@ char *vargv;
 FORWARD _PROTOTYPE(char *proc_name, (int proc_nr));
 
 /*===========================================================================*
- *				DEBUG routines				     *
+ *				p_dmp    				     *
  *===========================================================================*/
 #if (CHIP == INTEL)
 PUBLIC void p_dmp()
@@ -54,11 +54,12 @@ PUBLIC void p_dmp()
   if (rp == END_PROC_ADDR) rp = BEG_PROC_ADDR; else printf("--more--\r");
   oldrp = rp;
 }
-
 #endif				/* (CHIP == INTEL) */
 
+/*===========================================================================*
+ *				map_dmp    				     *
+ *===========================================================================*/
 #if (SHADOWING == 0)
-
 PUBLIC void map_dmp()
 {
   register struct proc *rp;
@@ -118,6 +119,9 @@ PUBLIC void map_dmp()
 #if (CHIP == M68000)
 FORWARD _PROTOTYPE(void mem_dmp, (char *adr, int len));
 
+/*===========================================================================*
+ *				p_dmp    				     *
+ *===========================================================================*/
 PUBLIC void p_dmp()
 {
 /* Proc table dump */
@@ -159,6 +163,9 @@ PUBLIC void p_dmp()
 }
 
 
+/*===========================================================================*
+ *				reg_dmp    				     *
+ *===========================================================================*/
 PUBLIC void reg_dmp(rp)
 struct proc *rp;
 {
@@ -189,6 +196,9 @@ struct proc *rp;
 }
 
 
+/*===========================================================================*
+ *				mem_dmp    				     *
+ *===========================================================================*/
 PRIVATE void mem_dmp(adr, len)
 char *adr;
 int len;
@@ -211,6 +221,9 @@ int len;
 #endif				/* (CHIP == M68000) */
 
 
+/*===========================================================================*
+ *				proc_name    				     *
+ *===========================================================================*/
 PRIVATE char *proc_name(proc_nr)
 int proc_nr;
 {
