@@ -1,6 +1,6 @@
 #include <lib.h>
 #define mkfifo	_mkfifo
-#define mknod4	_mknod4
+#define mknod	_mknod
 #include <sys/stat.h>
 #include <unistd.h>
 
@@ -8,5 +8,5 @@ PUBLIC int mkfifo(name, mode)
 _CONST char *name;
 Mode_t mode;
 {
-  return mknod4(name, mode | S_IFIFO, (Dev_t) 0, (long) 0);
+  return mknod(name, mode | S_IFIFO, (Dev_t) 0);
 }
