@@ -1,3 +1,8 @@
+/* Many of the tests require 1.6.n, n > 16, so we may as well assume that
+ * POSIX signals are implemented.
+ */
+#define SIGACTION
+
 /* test29: read(), write()	Author: Jan-Mark Wams (jms@cs.vu.nl) */
 
 #include <sys/types.h>
