@@ -31,7 +31,7 @@ char buf[ITEMS] = {0,1,2,3,4,5,6,7,8,9,8,7,6,5,4,3,2,1,0,1,2,3,4,5,6,7,8,9};
 
 int subtest, errct, xfd;
 int whence = SEEK_SET, func_code = F_SETLK;
-extern char *environ;
+extern char **environ;
 
 _PROTOTYPE(int main, (int argc, char *argv []));
 _PROTOTYPE(void test7a, (void));
@@ -389,8 +389,18 @@ void test7f()
 	if (wait(&s) != pid) e(7);
 	if ( (s>>8) != 19) e(8);
   } else {
-	/* Child tries to lock and should block. */
-	signal(SIGTERM, sigfunc);
+	/* Child tries to lock and should block.
+	 * `signal(SIGTERM, sigfunc);' to set the signal handler is inadequate
+	 * because on systems like BSD the sigaction flags for signal include
+	 * `SA_RESTART' so syscalls are restarted after they have been
+	 * interrupted by a signal.
+	 */
+	struct sigaction sa, osa;
+
+	sa.sa_handler = sigfunc;
+	sigemptyset(&sa.sa_mask);
+	sa.sa_flags = 0;
+	if (sigaction(SIGTERM, &sa, &osa) < 0) e(999);
 	if (creat("T7.f1", 0777) < 0) e(9);
 	func_code = F_SETLKW;
 	if (set(WRITE, 0, 3) != -1) e(10);	/* should block */
@@ -630,15 +640,26 @@ int b;
 /* Test to see if byte b is locked.  Return L or U */
 {
   struct flock flock;
-  int r;
+  pid_t pid;
+  int status;
 
   flock.l_type = F_WRLCK;
   flock.l_whence = whence;
   flock.l_start = (long) b;
   flock.l_len = 1;
-  r = fcntl(xfd, F_GETLK, &flock);
-  if (r != 0) e(1000);
-  return(flock.l_type == F_UNLCK ? U : L);  
+
+  /* Process' own locks are invisible to F_GETLK, so fork a child to test. */
+  pid = fork();
+  if (pid == 0) {
+	if (fcntl(xfd, F_GETLK, &flock) != 0) e(2000);
+	exit(flock.l_type == F_UNLCK ? U : L);  
+  }
+  if (pid == -1) e(2001);
+  if (fcntl(xfd, F_GETLK, &flock) != 0) e(2002);
+  if (flock.l_type != F_UNLCK) e(2003);
+  if (wait(&status) != pid) e(2004);
+  if (!WIFEXITED(status)) e(2005);
+  return(WEXITSTATUS(status));
 }
 
 void e(n)
